/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data IO Daemon module: IFileResolve Interface
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: $
 *  Release: $Name:  $
 *
 */

#ifndef GLITE_DATA_IO_DAEMON_IFILERESOLVE_HXX_
#define GLITE_DATA_IO_DAEMON_IFILERESOLVE_HXX_

#include "glite/data/io/daemon/FileItem.hxx"
#include "glite/data/io/daemon/UserPrincipal.hxx"
#include "glite/data/io/daemon/FileStat.hxx"

namespace glite  {
namespace io     {
namespace resolve{
    
/**
 * Base class for the File Resolve plugin. To create a plugin, you must 
 * implement that interface and register it to the FileResolveRepository
 * @interface
 */
class IFileResolve {
public:

    /**
     * IFileResolve Context Class
     * @stereotype bean
     */
    class Context{
    public:
        /**
         * Get File Item Property
         * @return File Item Property
         */
        FileItem * getFileItem() const {return _fileItem;}
        /**
         * Get User Principal Property
         * @return User Principal Property
         */
        const UserPrincipal * getUserPrincipal() const {return _user;}

        /**
         * Get The context Id
         * @return Context id
         */
        int getContextId() const {return _id;}
        
        /**
         * Set The context Id
         * @param i [IN]the Context id value
         */
        void setContextId(int i){_id = i;}

        /**
         * Get the error message string
         * @return a string with the error description, if any
         * @return an empty string ("") if no error detected
         */
         virtual std::string  getErrorMessage() const {return _error_message;}

        /**
         * Clear the error message string
         * @makes the error message string to be ""
         */
         virtual void clearErrorMessage() {_error_message.clear();}

        /**
         * Set the error message string
         * @_error_message - the string describing an error
         */
    virtual void setErrorMessage(std::string error){_error_message = error;}
        
    protected: 

      friend class IFileResolve;
        /**
         * Constructor
         * @param f [IN] the File Item Property
         * @param u [IN] the User Principal Property
         */
        Context(FileItem * f, const UserPrincipal * u):_fileItem(f),_user(u),_id(-1){_error_message = "";}
        /**
         * Destructor
         */
        virtual ~Context(){}

     
    private: 
        /*
         * Default Constructor Not Implemented
         */
        Context();
       
        /*
         * the File Item Property
         */
        FileItem * _fileItem;
        /*
         * the User Principal Property
         */
        const UserPrincipal * _user;
    
        /**
         * Context Id
         */
        int _id;

        /*
         * the error message string
         */
        std::string _error_message;

    };

    /**
     * Get the Plug-in name
     * @return the Plug-in name
     */
    virtual const char * getType() const = 0;

    /**
     * Initialize The Plugin Context
     * @param f [INOUT] the FileItem object that contains the file information. 
     * This parameter could be modified by some IFIleResolve Calls
     * @param u [IN] the UserPrincipal object that contains user information. It 
     * may be used for delegating credentials
     * @return the Initialized Context or 0 if failed
     */
    virtual IFileResolve::Context * initContext(FileItem * f, const UserPrincipal * u) = 0;

    /**
     * Finalize The Plugin Context
     * @param ctx [IN] the Plugin Context
     */
    virtual void releaseContext(IFileResolve::Context * ctx) = 0;

    /**
     * Set the FileItem GUID property starting from its LFN
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int resolveGUID(IFileResolve::Context * ctx) = 0;

	/**
     * Get the FileItem Stat properties
     * @param ctx [IN] the Plugin Context containing the File and User 
     * Information
     * @param file_stats [OUT] the object where to store the file statistics 
     * @return 0 if success
     */
    virtual int getStats(IFileResolve::Context * ctx, FileStat * file_stats) = 0;

    /**
     * Set the FileItem SURL property from its GUID
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int resolveName(IFileResolve::Context * ctx) = 0;

    /**
     * Do a reservation based on GUID. This methods will generate a new GUID 
     * if is not provided and also a new SURL based on the GUID or on the given 
     * LFN (if any). This method requires that commitGUID or cancelGUID is 
     * invoke afterward.
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int reserveGUID(IFileResolve::Context * ctx) = 0;

    /**
     * Generate a new TURL
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int generateName(IFileResolve::Context * ctx) = 0;

    /**
     * Commit a previous reservation
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int commit(IFileResolve::Context * ctx) = 0;

    /**
     * Cancel a previous reservation
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int cancel(IFileResolve::Context * ctx) = 0;

	/**
     * Delete a File
     * @param ctx [INOUT] the Plugin Context containing the File and User 
     * Information
     * @return 0 if success
     */
    virtual int unlink(IFileResolve::Context * ctx){
		// Allows old implementation to compile
		return -1;
	}

private:

    /** @link dependency */
    /*# Context lnkContext; */
};


} // End namespace resolve
} // End namespace io
} // End namespace glite
    

#endif //GLITE_DATA_IO_DAEMON_IFILERESOLVE_HXX_
